package com.android.td5;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class SieveSurfaceView extends SurfaceView implements SurfaceHolder.Callback {
	public class ThreadSurfaceView extends Thread {
    	private SurfaceHolder sieveSurfaceHolder;

    	private boolean running = true;

    	private boolean refresh= true;
    	
    	ThreadSurfaceView(SurfaceHolder sieveSurfaceHolder) {
    		this.sieveSurfaceHolder = sieveSurfaceHolder;
    	}
    	
    	public void setRunning(boolean running) {
    		this.running = running;
    	}

    	public void setRefresh(boolean refresh) {
    		this.refresh = refresh;
    	}
    	
    	public void run() {
    		// To be completed
    	}
    }
	
	// Paint/Colors
    private final Paint TRUE_PAINT = new Paint();
    private final Paint FALSE_PAINT = new Paint();

    // Mark size
    private int size = 16;
    
    // Surface information
    private int width;
    private int height;
    
    private int round(int v) {
    	return (v / size) * size;
    }
    
    private SieveModel model;
	private SurfaceHolder sieveSurfaceHolder;
	private ThreadSurfaceView threadSurfaceView;

	public ThreadSurfaceView getThread() {
		return threadSurfaceView;
	}
	
	public void setSieveModel(SieveModel model) {
		this.model = model;
	}
	
	// Erase the rectangular canvas
	private void eraseAll(Canvas c) {
    	c.drawColor(Color.BLACK);
    }
	
    // Draw mark in the rectangular canvas at the right position
    private void drawInteger(Canvas c, int i, boolean b) {
    	int left = (i % (width / size)) * size;
    	int top = (i / (width / size)) * size;
    	int right = left + size - 1;
    	int bottom = top + size - 1;
    	c.drawRect(left, top, right, bottom, b ? TRUE_PAINT : FALSE_PAINT);
    }

    private void drawAll(Canvas c) {
    	boolean isPrime[] = model.isPrime;
    	    	
    	if (isPrime != null) {
        	double d = Math.sqrt((double)width*height / isPrime.length);
        	size = Math.max(2, Math.min(32, (int)d));
      
    		for (int i = 0; i < isPrime.length && i < width*height / (size*size); i++)
    			drawInteger(c, i, isPrime[i]);
    	}
    }

    public SieveSurfaceView(Context context, AttributeSet attrs) {
        super(context, attrs);

        // register our interest in hearing about changes to our surface
        sieveSurfaceHolder = getHolder();
        sieveSurfaceHolder.addCallback(this);
        
        TRUE_PAINT.setColor(Color.GREEN);
        FALSE_PAINT.setColor(Color.DKGRAY);
    }
    
    /* Pause on focus lost. e.g. user switches to take a call. */
    @Override
    public void onWindowFocusChanged(boolean hasWindowFocus) {
    	threadSurfaceView.setRefresh(hasWindowFocus);
    }
    
    /* Callback invoked when the surface dimensions change. */
    @Override
    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height) {
    	this.width = round(width);
    	this.height = round(height);
    }
    
    /* Callback invoked when the Surface has been created and is ready to be used. */
    @Override
    public void surfaceCreated(SurfaceHolder holder) {
    	Canvas c = sieveSurfaceHolder.lockCanvas();
    	width = round(c.getWidth());
    	height = round(c.getHeight());
    	sieveSurfaceHolder.unlockCanvasAndPost(c);
    	
        threadSurfaceView = new ThreadSurfaceView(sieveSurfaceHolder);
        threadSurfaceView.start();
    }

    /*
     * Callback invoked when the Surface has been destroyed.
     * After this method returns, the Surface/Canvas must
     * never be touched again.
     */
    @Override
    public void surfaceDestroyed(SurfaceHolder holder) {
    	// Tell thread to shut down, and wait for it to finish, or else
        // it might touch the SurfaceView after the function returns
        boolean retry = true;
        threadSurfaceView.setRunning(false);
        while (retry) {
            try {
                threadSurfaceView.join();
                retry = false;
            } catch (InterruptedException e) {
            	// Retry
            }
        }
    }
}
